@@ -9,7 +9,7 @@ static const double jump_speed = 291 + (2. / 3.);
 static const double jump_acc = 1458 + (1. / 3.);
 static const double boost_consumption = 33. + (1. / 3.);
 static const double max_speed = 2300;
-static const double simulation_dt = 1. / 30.;
+static const double simulation_dt = 1. / 20.;
 static const double angle_resistance_multiplier = 50;
 
 // Vector stuff
@@ -148,6 +148,12 @@ struct jump_shot
     struct post_correction targets;
 };
 
+struct ground_shot
+{
+	_Bool found;
+	struct post_correction targets;
+};
+
 struct aerial_shot
 {
     int found;
@@ -213,7 +219,7 @@ double throttle_acceleration(double car_velocity_x)
     return y0 + ((y1 - y0) / (x1 - x0)) * (car_velocity_x - x0);
 }
 
-double car_drive_to_target_simulation(double boost_accel, double car_to_target, double car_speed, int car_boost, double resistance)
+double car_drive_to_target_simulation(double boost_accel, double car_to_target, double car_speed, int car_boost)
 {
     double b = car_boost;
     double t = 0;
@@ -224,8 +230,6 @@ double car_drive_to_target_simulation(double boost_accel, double car_to_target,
     {
         if (v < 1410)
             v += throttle_acceleration(v) * simulation_dt;
-        else
-            v -= resistance * simulation_dt;
 
         if (b > boost_consumption * simulation_dt && v < max_speed - (boost_accel * simulation_dt))
         {
@@ -246,11 +250,12 @@ double car_turn_simulation(double turning_time, double car_speed)
 {
     double t = turning_time;
     double v = car_speed;
+    double r = 100 * simulation_dt;
 
     while (t >= simulation_dt)
     {
         if (v > 1410)
-            v -= 100 * simulation_dt;
+            v -= t;
 
         v += throttle_acceleration(v) * simulation_dt;
         t -= simulation_dt;
@@ -259,6 +264,31 @@ double car_turn_simulation(double turning_time, double car_speed)
     return v;
 }
 
+double get_jump_time(double car_to_target_z, double car_z_velocity, double gravity_z) {
+    double t = 0;
+    double l = 0;
+    double v = car_z_velocity;
+    double g = gravity_z * simulation_dt;
+    double js = jump_speed * simulation_dt;
+    double ja = jump_acc * simulation_dt;
+
+    while (car_to_target_z - l > 0)
+    {
+        if (l > car_to_target_z)
+            return -1;
+
+        if (t == 0)
+            v += js;
+
+        v += ja;
+        v -= g;
+        l += v * simulation_dt;
+        t += simulation_dt;
+    }
+
+    return t;
+}
+
 // Main lib
 
 int find_landing_plane(Vector car_location, Vector car_velocity, double gravity)
@@ -302,8 +332,7 @@ struct jump_viable_dat jump_is_viable(double T, double boost_accel, double dista
 {
     struct jump_viable_dat r;
 
-    if (multiply(direction, double_to_vector(distance)).z > 300)
-    {
+    if (multiply(direction, double_to_vector(distance)).z > 300) {
         r.forward = r.backward = 0;
         return r;
     }
@@ -311,8 +340,14 @@ struct jump_viable_dat jump_is_viable(double T, double boost_accel, double dista
     double forward_angle = angle(flatten(direction), flatten(car_forward));
     double backward_angle = Py_MATH_PIl - forward_angle;
 
-    r.forward = T > car_drive_to_target_simulation(boost_accel, distance, car_speed, car_boost, forward_angle * angle_resistance_multiplier);
-    r.backward = distance < 1500 && T > car_drive_to_target_simulation(boost_accel, distance, car_speed, 0, backward_angle * angle_resistance_multiplier);
+    double forward_turn_time = forward_angle * 0.418;
+    double backward_turn_time = backward_angle * 0.418;
+
+    double forward_time = T - forward_turn_time;
+    double backward_time = T - backward_turn_time;
+
+    r.forward = forward_time > 0 && forward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(forward_turn_time, car_speed), car_boost);
+    r.backward = backward_time > 0 && distance < 1500 && backward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(backward_turn_time, car_speed), 0);
 
     return r;
 }
@@ -379,7 +414,10 @@ _Bool double_jump_is_viable(double T, double boost_accel, double distance, Vecto
     if (z < 380 || z > 490)
         return 0;
 
-    return T > car_drive_to_target_simulation(boost_accel, distance, car_speed, car_boost, angle(flatten(direction), flatten(car_forward)) * angle_resistance_multiplier);
+    double forward_turn_time = angle(flatten(direction), flatten(car_forward)) * 0.418;
+    double forward_time = T - forward_turn_time;
+
+    return forward_time > 0 && forward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(forward_turn_time, car_speed), car_boost);
 }
 
 struct jump_shot parse_slice_for_double_jump_with_target(double time_remaining, double best_shot_value, Vector ball_location, Vector car_location, Vector car_forward, int car_boost, Vector left_target, Vector right_target, double boost_accel, double car_speed)
@@ -428,10 +466,74 @@ struct car
     Vector velocity;
     Vector up;
     Vector forward;
+    Vector hitbox;
     int airborne;
     int boost;
 };
 
+_Bool ground_shot_is_viable(double T, Vector hitbox, double boost_accel, double distance, Vector direction, Vector car_forward, int car_boost, double car_speed)
+{
+    if (multiply(direction, double_to_vector(distance)).z > 92.75 + hitbox.z)
+        return 0;
+
+    distance = distance - 92.75 - hitbox.x / 2.;
+
+    double forward_angle = angle(flatten(direction), flatten(car_forward));
+    double backward_angle = Py_MATH_PIl - forward_angle;
+
+    double forward_turn_time = forward_angle * 0.418;
+    double backward_turn_time = backward_angle * 0.418;
+
+    double forward_time = T - forward_turn_time;
+    double backward_time = T - backward_turn_time;
+
+    _Bool forward = forward_time > 0 && forward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(forward_turn_time, car_speed), car_boost);
+    _Bool backward = backward_time > 0 && distance < 1500 && backward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(backward_turn_time, car_speed), 0);
+
+    return forward || backward;
+}
+
+struct ground_shot parse_slice_for_ground_shot_with_target(double time_remaining, double best_shot_value, double boost_accel, Vector ball_location, struct car me, Vector left_target, Vector right_target)
+{
+	struct ground_shot r;
+	r.found = 0;
+
+    Vector car_to_ball = sub(ball_location, me.location);
+    struct opti_norm car_to_ball_norm = normalize(car_to_ball);
+    Vector direction = car_to_ball_norm.vector;
+
+    r.targets = correct_for_posts(ball_location, left_target, right_target);
+    if (!r.targets.swapped)
+    {
+        Vector left_vector = normalize(sub(r.targets.left, ball_location)).vector;
+        Vector right_vector = normalize(sub(r.targets.right, ball_location)).vector;
+        Vector best_shot_vector = clamp2D(direction, left_vector, right_vector);
+
+        if (in_field(sub(ball_location, multiply(best_shot_vector, double_to_vector(best_shot_value))), 1) && find_slope(best_shot_vector, car_to_ball) > 0)
+            r.found = ground_shot_is_viable(time_remaining, me.hitbox, boost_accel, car_to_ball_norm.magnitude, direction, me.forward, me.boost, magnitude(me.velocity));
+    }
+
+    return r;
+};
+
+struct ground_shot parse_slice_for_ground_shot(double time_remaining, double best_shot_value, double boost_accel, Vector ball_location, struct car me)
+{
+    struct ground_shot r;
+    r.found = 0;
+
+    Vector car_to_ball = sub(ball_location, me.location);
+    struct opti_norm car_to_ball_norm = normalize(car_to_ball);
+    Vector direction = car_to_ball_norm.vector;
+
+    Vector best_shot_vector = direction;
+
+    if (in_field(sub(ball_location, multiply(best_shot_vector, double_to_vector(best_shot_value))), 1) && find_slope(best_shot_vector, car_to_ball) > 0)
+        r.found = ground_shot_is_viable(time_remaining, me.hitbox, boost_accel, car_to_ball_norm.magnitude, direction, me.forward, me.boost, magnitude(me.velocity));
+
+    return r;
+};
+
+
 struct aerial_shot aerial_is_viable(double time_remaining, double hitbox_height, double boost_accel, Vector gravity, struct car me, Vector target)
 {
     struct aerial_shot r;
@@ -440,7 +542,7 @@ struct aerial_shot aerial_is_viable(double time_remaining, double hitbox_height,
     Vector xf = add(add(me.location, multiply(me.velocity, T)), multiply(multiply(multiply(double_to_vector(0.5), gravity), T), T));
     Vector vf = add(me.velocity, multiply(gravity, T));
 
-    _Bool ceiling = me.location.z > 2044 - hitbox_height;
+    _Bool ceiling = me.location.z > 2044 - (hitbox_height * 2);
 
     if (me.airborne == -1 && !ceiling)
     {
@@ -468,8 +570,8 @@ struct aerial_shot aerial_is_viable(double time_remaining, double hitbox_height,
     _Bool enough_time = fabs(ratio) < 0.95;
     _Bool enough_speed = magnitude(velocity_estimate) < 0.95 * max_speed;
 
-    r.fast = enough_speed && enough_boost && enough_time;
-    r.found = r.fast;
+    r.found = enough_speed && enough_boost && enough_time;
+    r.fast = 1;
 
     if (!me.airborne && !ceiling && time_remaining < 1.45)
     {
@@ -527,7 +629,7 @@ struct aerial_shot parse_slice_for_aerial_shot_with_target(double time_remaining
             double slope = find_slope(r.ball_intercept, car_to_ball);
             if (slope > 0.5)
             {
-                struct aerial_shot v = aerial_is_viable(time_remaining, 144, boost_accel, gravity, me, r.ball_intercept);
+                struct aerial_shot v = aerial_is_viable(time_remaining, me.hitbox.z, boost_accel, gravity, me, r.ball_intercept);
                 r.found = v.found;
                 r.fast = v.fast;
             }
@@ -552,7 +654,7 @@ struct aerial_shot parse_slice_for_aerial_shot(double time_remaining, double bes
         double slope = find_slope(r.ball_intercept, car_to_ball);
         if (slope > 0.5)
         {
-            struct aerial_shot v = aerial_is_viable(time_remaining, 144, boost_accel, gravity, me, r.ball_intercept);
+            struct aerial_shot v = aerial_is_viable(time_remaining, me.hitbox.z, boost_accel, gravity, me, r.ball_intercept);
             r.found = v.found;
             r.fast = v.fast;
         }
@@ -569,10 +671,10 @@ static PyObject *method_parse_slice_for_jump_shot_with_target(PyObject *self, Py
     double time_remaining, best_shot_value, boost_accel, car_speed, cap_;
     Vector ball_location, car_location, car_forward, left_target, right_target;
 
-    // new args are for >=1.8
+    // args are for >=1.8
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)id(ddd)(ddd)", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z))
     {
-        // old args are for <1.8 and >=1.7
+        // old legacy args are for <1.8 and >=1.7
         // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
         PyErr_Clear();
         if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)i(ddd)(ddd)d", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z, &cap_))
@@ -593,10 +695,10 @@ static PyObject *method_parse_slice_for_jump_shot(PyObject *self, PyObject *args
     double time_remaining, best_shot_value, boost_accel, car_speed, cap_;
     Vector ball_location, car_location, car_forward;
 
-    // new args are for >=1.8
+    // args are for >=1.8
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)id", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed))
     {
-        // old args are for <1.8 and >=1.7
+        // old legacy args are for <1.8 and >=1.7
         // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
         PyErr_Clear();
         if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)i|d", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &cap_))
@@ -616,6 +718,7 @@ static PyObject *method_jump_shot_is_viable(PyObject *self, PyObject *args)
     double time_remaining, boost_accel, car_speed, distance;
     Vector car_forward, direction;
 
+    // args are for all versions, so far
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)id", &time_remaining, &boost_accel, &distance, &direction.x, &direction.y, &direction.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed))
         return NULL;
 
@@ -630,10 +733,10 @@ static PyObject *method_parse_slice_for_double_jump_with_target(PyObject *self,
     double time_remaining, best_shot_value, boost_accel, car_speed, cap_;
     Vector ball_location, car_location, car_forward, left_target, right_target;
 
-    // new args are for >=1.8
+    // args are for >=1.8
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)id(ddd)(ddd)", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z))
     {
-        // old args are for <1.8 and >=1.7
+        // old legacy args are for <1.8 and >=1.7
         // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
         PyErr_Clear();
         if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)i(ddd)(ddd)|d", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z, &cap_))
@@ -654,10 +757,10 @@ static PyObject *method_parse_slice_for_double_jump(PyObject *self, PyObject *ar
     double time_remaining, best_shot_value, boost_accel, car_speed, cap_;
     Vector ball_location, car_location, car_forward;
 
-    // new args are for >=1.8
+    // args are for >=1.8
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)id", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed))
     {
-        // old args are for <1.8 and >=1.7
+        // old legacy args are for <1.8 and >=1.7
         // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
         PyErr_Clear();
         if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)i|d", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &cap_))
@@ -677,7 +780,8 @@ static PyObject *method_double_jump_shot_is_viable(PyObject *self, PyObject *arg
     int car_boost;
     double time_remaining, boost_accel, car_speed, distance;
     Vector car_forward, direction;
-
+    
+    // args are for all versions, so far
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)id", &time_remaining, &boost_accel, &distance, &direction.x, &direction.y, &direction.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &car_speed))
         return NULL;
 
@@ -686,15 +790,68 @@ static PyObject *method_double_jump_shot_is_viable(PyObject *self, PyObject *arg
     return (shot_viable) ? Py_True : Py_False;
 }
 
+static PyObject *method_parse_slice_for_ground_shot_with_target(PyObject *self, PyObject *args)
+{
+    struct car me;
+    double time_remaining, best_shot_value, boost_accel;
+    Vector ball_location, left_target, right_target;
+
+	// args are for >= 1.10
+	if (!PyArg_ParseTuple(args, "ddd(ddd)((ddd)(ddd)(ddd)(ddd)(ddd)ii)(ddd)(ddd)", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.hitbox.x, &me.hitbox.y, &me.hitbox.z, &me.airborne, &me.boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z))
+		return NULL;
+
+    struct ground_shot data_struct = parse_slice_for_ground_shot_with_target(time_remaining, best_shot_value, boost_accel, ball_location, me, left_target, right_target);
+
+    return Py_BuildValue("{s:i,s:((ddd)(ddd))}", "found", data_struct.found, "targets", data_struct.targets.left.x, data_struct.targets.left.y, data_struct.targets.left.z, data_struct.targets.right.x, data_struct.targets.right.y, data_struct.targets.right.z);
+};
+
+static PyObject *method_parse_slice_for_ground_shot(PyObject *self, PyObject *args)
+{
+    struct car me;
+    double time_remaining, best_shot_value, boost_accel;
+    Vector ball_location;
+
+	// args are for >= 1.10
+	if (!PyArg_ParseTuple(args, "ddd(ddd)((ddd)(ddd)(ddd)(ddd)(ddd)ii)", &time_remaining, &best_shot_value, &boost_accel, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.hitbox.x, &me.hitbox.y, &me.hitbox.z, &me.airborne, &me.boost))
+		return NULL;
+
+    struct ground_shot data_struct = parse_slice_for_ground_shot(time_remaining, best_shot_value, boost_accel, ball_location, me);
+
+    return Py_BuildValue("{s:i}", "found", data_struct.found);
+};
+
+static PyObject *method_ground_shot_is_viable(PyObject *self, PyObject *args)
+{
+	int car_boost;
+    double time_remaining, boost_accel, distance, car_speed;
+    Vector direction, car_forward, hitbox;
+
+    // args are for >= 1.10
+    if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)id", &time_remaining, &boost_accel, &distance, &direction.x, &direction.y, &direction.z, &car_forward.x, &car_forward.y, &car_forward.z, &hitbox.x, &hitbox.y, &hitbox.z, &car_boost, &car_speed))
+        return NULL;
+
+    _Bool shot_viable = ground_shot_is_viable(time_remaining, hitbox, boost_accel, distance, direction, car_forward, car_boost, car_speed);
+
+    return (shot_viable) ? Py_True : Py_False;
+}
+
 static PyObject *method_parse_slice_for_aerial_shot_with_target(PyObject *self, PyObject *args)
 {
     struct car me;
     double time_remaining, best_shot_value, boost_accel, cap_;
     Vector gravity, ball_location, left_target, right_target;
 
-    // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
-    if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)ii)(ddd)(ddd)|d", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.airborne, &me.boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z, &cap_))
-        return NULL;
+	// new args are for >= 1.10
+	if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)(ddd)ii)(ddd)(ddd)", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.hitbox.x, &me.hitbox.y, &me.hitbox.z, &me.airborne, &me.boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z))
+	{
+		// legacy args are for < 1.10
+	    // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
+		PyErr_Clear();
+	    if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)ii)(ddd)(ddd)|d", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.airborne, &me.boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z, &cap_))
+	        return NULL;
+
+	    me.hitbox = (Vector){118.01, 76.71, 30.3};
+	}
 
     struct aerial_shot data_struct = parse_slice_for_aerial_shot_with_target(time_remaining, best_shot_value, boost_accel, gravity, ball_location, me, left_target, right_target);
 
@@ -707,9 +864,17 @@ static PyObject *method_parse_slice_for_aerial_shot(PyObject *self, PyObject *ar
     double time_remaining, best_shot_value, boost_accel, cap_;
     Vector gravity, ball_location;
 
-    // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
-    if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)ii)|d", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.airborne, &me.boost, &cap_))
-        return NULL;
+	// new args are for >= 1.10
+	if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)(ddd)ii)", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.hitbox.x, &me.hitbox.y, &me.hitbox.z, &me.airborne, &me.boost))
+	{
+		// legacy args are for < 1.10
+	    // cap_ is no longer needed, but a lot of instances still pass it in as an argument; this is why it's optional
+		PyErr_Clear();
+	    if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)((ddd)(ddd)(ddd)(ddd)ii)|d", &time_remaining, &best_shot_value, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &ball_location.x, &ball_location.y, &ball_location.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.airborne, &me.boost, &cap_))
+	        return NULL;
+
+	    me.hitbox = (Vector){118.01, 76.71, 30.3};
+	}
 
     struct aerial_shot data_struct = parse_slice_for_aerial_shot(time_remaining, best_shot_value, boost_accel, gravity, ball_location, me);
 
@@ -722,6 +887,7 @@ static PyObject *method_aerial_shot_is_viable(PyObject *self, PyObject *args)
     Vector gravity, target;
     struct car me;
 
+    // args are for all versions, so far
     if (!PyArg_ParseTuple(args, "ddd(ddd)(ddd)(ddd)(ddd)(ddd)ii(ddd)", &time_remaining, &hitbox_height, &boost_accel, &gravity.x, &gravity.y, &gravity.z, &me.location.x, &me.location.y, &me.location.z, &me.velocity.x, &me.velocity.y, &me.velocity.z, &me.up.x, &me.up.y, &me.up.z, &me.forward.x, &me.forward.y, &me.forward.z, &me.airborne, &me.boost, &target.x, &target.y, &target.z))
         return NULL;
 
@@ -732,10 +898,10 @@ static PyObject *method_aerial_shot_is_viable(PyObject *self, PyObject *args)
 
 static PyObject *method_find_landing_plane(PyObject *self, PyObject *args)
 {
-    // int find_landing_plane(Vector car_location, Vector car_velocity, double gravity)
     Vector car_location, car_velocity;
     double gravity;
 
+    // args are for all versions, so far
     if (!PyArg_ParseTuple(args, "(ddd)(ddd)d", &car_location.x, &car_location.y, &car_location.z, &car_velocity.x, &car_velocity.y, &car_velocity.z, &gravity))
         return NULL;
 
@@ -744,17 +910,33 @@ static PyObject *method_find_landing_plane(PyObject *self, PyObject *args)
     return Py_BuildValue("i", landing_plane);
 }
 
+static PyObject *method_get_jump_time(PyObject *self, PyObject *args)
+{
+    double car_to_target_z, car_z_velocity, gravity_z;
+
+    if (!PyArg_ParseTuple(args, "ddd", &car_to_target_z, &car_z_velocity, &gravity_z))
+        return NULL;
+
+    double time = get_jump_time(car_to_target_z, car_z_velocity, gravity_z);
+
+    return Py_BuildValue("d", time);
+}
+
 static PyMethodDef methods[] = {
     {"parse_slice_for_jump_shot_with_target", method_parse_slice_for_jump_shot_with_target, METH_VARARGS, "Parse slice for a jump shot with a target"},
     {"parse_slice_for_jump_shot", method_parse_slice_for_jump_shot, METH_VARARGS, "Parse slice for a jump shot"},
-    {"jump_shot_is_viable", method_jump_shot_is_viable, METH_VARARGS, "Check if an aerial is viable"},
+    {"jump_shot_is_viable", method_jump_shot_is_viable, METH_VARARGS, "Check if an jump_shot is viable"},
     {"parse_slice_for_double_jump_with_target", method_parse_slice_for_double_jump_with_target, METH_VARARGS, "Parse slice for a double jump with a target"},
     {"parse_slice_for_double_jump", method_parse_slice_for_double_jump, METH_VARARGS, "Parse slice for a double jump"},
-    {"double_jump_shot_is_viable", method_double_jump_shot_is_viable, METH_VARARGS, "Check if an aerial is viable"},
+    {"double_jump_shot_is_viable", method_double_jump_shot_is_viable, METH_VARARGS, "Check if an double jump is viable"},
     {"parse_slice_for_aerial_shot_with_target", method_parse_slice_for_aerial_shot_with_target, METH_VARARGS, "Parse slice for an aerial shot with a target"},
     {"parse_slice_for_aerial_shot", method_parse_slice_for_aerial_shot, METH_VARARGS, "Parse slice for an aerial shot"},
     {"aerial_shot_is_viable", method_aerial_shot_is_viable, METH_VARARGS, "Check if an aerial is viable"},
+    {"parse_slice_for_ground_shot_with_target", method_parse_slice_for_ground_shot_with_target, METH_VARARGS, "Parse slice for a ground shot with a target"},
+    {"parse_slice_for_ground_shot", method_parse_slice_for_ground_shot, METH_VARARGS, "Parse slice for a ground shot"},
+    {"ground_shot_is_viable", method_ground_shot_is_viable, METH_VARARGS, "Check if a ground shot is viable"},
     {"find_landing_plane", method_find_landing_plane, METH_VARARGS, "Find the plane (side wall, back wall, ceiling, or floor) that the car will collid with first"},
+    {"get_jump_time", method_get_jump_time, METH_VARARGS, "Get the time required to jump and reach at target height"},
     {NULL, NULL, 0, NULL}};
 
 static struct PyModuleDef module = {
